==> ml_algo-15.6.5.tar.gz: test/tree_trainer/split_selector/greedy_split_selector_test.dart <==
import 'package:ml_algo/src/tree_trainer/split_selector/greedy_split_selector.dart';
import 'package:ml_linalg/matrix.dart';
import 'package:mockito/mockito.dart';
import 'package:test/test.dart';

import '../../mocks.dart';

void main() {
  group('GreedyTreeSplitSelector', () {
    test('should find the best split', () {
      final samples = Matrix.fromList([
        [10, 20, 30, 40, 1],
        [12, 22, 32, 42, 2],
        [11, 21, 31, 41, 3],
      ]);

      final targetColIdx = 4;

      final worstSplitIdx = 0;
      final worseSplitIdx = 2;
      final goodSplitIdx = 3;
      final bestSplitIdx = 1;

      final worstSplit = {
        TreeNodeMock(): Matrix.fromList([[1, 2, 3, 4, -1, -1, -1]]),
        TreeNodeMock(): Matrix.fromList([[10, 20, 30, 40, -1, -1, -1]]),
      };

      final worseSplit = {
        TreeNodeMock(): Matrix.fromList([[5, 6, 7, 8, -1, -1, 0]]),
        TreeNodeMock(): Matrix.fromList([[50, 60, 70, 80, -1, -1, 0]]),
      };

      final goodSplit = {
        TreeNodeMock(): Matrix.fromList([
          [15, 16, 17, 18, -1, 0, 0],
        ]),
        TreeNodeMock(): Matrix.fromList([
          [150, 160, 170, 180, -1, 0, 0],
        ]),
      };

      final bestSplit = {
        TreeNodeMock(): Matrix.fromList([
          [125, 126, 127, 128, 0, 0, 0],
        ]),
        TreeNodeMock(): Matrix.fromList([
          [1500, 1600, 1700, 1800, 0, 0, 0],
        ]),
      };

      final featuresIdxs = [
        worstSplitIdx, bestSplitIdx, worseSplitIdx, goodSplitIdx,
      ];
      final assessor = TreeSplitAssessorMock();
      final splitter = TreeSplitterMock();

      when(splitter.split(samples, worstSplitIdx, targetColIdx))
          .thenReturn(worstSplit);
      when(splitter.split(samples, worseSplitIdx, targetColIdx))
          .thenReturn(worseSplit);
      when(splitter.split(samples, goodSplitIdx, targetColIdx))
          .thenReturn(goodSplit);
      when(splitter.split(samples, bestSplitIdx, targetColIdx))
          .thenReturn(bestSplit);

      when(assessor.getAggregatedError(worstSplit.values,
          targetColIdx)).thenReturn(0.999);
      when(assessor.getAggregatedError(worseSplit.values,
          targetColIdx)).thenReturn(0.8);
      when(assessor.getAggregatedError(goodSplit.values,
          targetColIdx)).thenReturn(0.4);
      when(assessor.getAggregatedError(bestSplit.values,
          targetColIdx)).thenReturn(0.1);

      final selector = GreedyTreeSplitSelector(assessor, splitter);
      final actualSplit = selector
          .select(samples, targetColIdx, featuresIdxs);

      expect(actualSplit.keys, equals(bestSplit.keys));
      expect(actualSplit.values, equals(bestSplit.values));
    });

    test('should select columns for splitting according to given feature '
        'columns indices', () {
      final observations = Matrix.fromList([
        [10, 1, 50, 1],
        [12, 2, 52, 2],
        [11, 3, 53, 3],
      ]);

      final targetColIdx = 3;

      final goodFeatureColIdx = 0;
      final ignoredFeatureColIdx = 1;
      final bestFeatureColIdx = 2;

      final goodSplit = {
        TreeNodeMock(): Matrix.fromList([
          [1, 2, 3, 4, -1, -1, -1],
        ]),
        TreeNodeMock(): Matrix.fromList([
          [10, 20, 30, 40, -1, -1, -1],
        ]),
      };

      final bestSplit = {
        TreeNodeMock(): Matrix.fromList([
          [15, 16, 17, 18, -1, 0, 0],
        ]),
        TreeNodeMock(): Matrix.fromList([
          [150, 160, 170, 180, -1, 0, 0],
        ]),
      };

      final featuresIdxs = {goodFeatureColIdx, bestFeatureColIdx};

      final assessor = TreeSplitAssessorMock();
      final splitter = TreeSplitterMock();
      final categoricalValues = [1.0, 2.0, 3.0];
      final colIdxToUniqueValues = {ignoredFeatureColIdx: categoricalValues};

      when(splitter.split(observations, goodFeatureColIdx, targetColIdx))
          .thenReturn(goodSplit);
      when(splitter.split(observations, bestFeatureColIdx, targetColIdx))
          .thenReturn(bestSplit);

      when(assessor.getAggregatedError(goodSplit.values,
          targetColIdx)).thenReturn(0.51);
      when(assessor.getAggregatedError(bestSplit.values,
          targetColIdx)).thenReturn(0.1);

      final selector = GreedyTreeSplitSelector(assessor, splitter);
      final stump = selector.select(observations, targetColIdx,
          featuresIdxs, colIdxToUniqueValues);

      expect(stump.keys, equals(bestSplit.keys));
      expect(stump.values, equals(bestSplit.values));

      verifyNever(splitter.split(observations, ignoredFeatureColIdx,
          targetColIdx));
    });
  });
}

==> ml_algo-16.0.2.tar.gz: test/tree_trainer/split_selector/greedy_split_selector_test.dart <==
import 'package:ml_algo/src/tree_trainer/split_selector/greedy_split_selector.dart';
import 'package:ml_linalg/matrix.dart';
import 'package:mockito/mockito.dart';
import 'package:test/test.dart';

import '../../mocks.mocks.dart';

void main() {
  group('GreedyTreeSplitSelector', () {
    test('should find the best split', () {
      final samples = Matrix.fromList([
        [10, 20, 30, 40, 1],
        [12, 22, 32, 42, 2],
        [11, 21, 31, 41, 3],
      ]);

      final targetColIdx = 4;

      final worstSplitIdx = 0;
      final worseSplitIdx = 2;
      final goodSplitIdx = 3;
      final bestSplitIdx = 1;

      final worstSplit = {
        MockTreeNode(): Matrix.fromList([[1, 2, 3, 4, -1, -1, -1]]),
        MockTreeNode(): Matrix.fromList([[10, 20, 30, 40, -1, -1, -1]]),
      };

      final worseSplit = {
        MockTreeNode(): Matrix.fromList([[5, 6, 7, 8, -1, -1, 0]]),
        MockTreeNode(): Matrix.fromList([[50, 60, 70, 80, -1, -1, 0]]),
      };

      final goodSplit = {
        MockTreeNode(): Matrix.fromList([
          [15, 16, 17, 18, -1, 0, 0],
        ]),
        MockTreeNode(): Matrix.fromList([
          [150, 160, 170, 180, -1, 0, 0],
        ]),
      };

      final bestSplit = {
        MockTreeNode(): Matrix.fromList([
          [125, 126, 127, 128, 0, 0, 0],
        ]),
        MockTreeNode(): Matrix.fromList([
          [1500, 1600, 1700, 1800, 0, 0, 0],
        ]),
      };

      final featuresIdxs = [
        worstSplitIdx, bestSplitIdx, worseSplitIdx, goodSplitIdx,
      ];
      final assessor = MockTreeSplitAssessor();
      final splitter = MockTreeSplitter();

      when(splitter.split(samples, worstSplitIdx, targetColIdx))
          .thenReturn(worstSplit);
      when(splitter.split(samples, worseSplitIdx, targetColIdx))
          .thenReturn(worseSplit);
      when(splitter.split(samples, goodSplitIdx, targetColIdx))
          .thenReturn(goodSplit);
      when(splitter.split(samples, bestSplitIdx, targetColIdx))
          .thenReturn(bestSplit);

      when(assessor.getAggregatedError(worstSplit.values,
          targetColIdx)).thenReturn(0.999);
      when(assessor.getAggregatedError(worseSplit.values,
          targetColIdx)).thenReturn(0.8);
      when(assessor.getAggregatedError(goodSplit.values,
          targetColIdx)).thenReturn(0.4);
      when(assessor.getAggregatedError(bestSplit.values,
          targetColIdx)).thenReturn(0.1);

      final selector = GreedyTreeSplitSelector(assessor, splitter);
      final actualSplit = selector
          .select(samples, targetColIdx, featuresIdxs);

      expect(actualSplit.keys, equals(bestSplit.keys));
      expect(actualSplit.values, equals(bestSplit.values));
    });

    test('should select columns for splitting according to given feature '
        'columns indices', () {
      final observations = Matrix.fromList([
        [10, 1, 50, 1],
        [12, 2, 52, 2],
        [11, 3, 53, 3],
      ]);

      final targetColIdx = 3;

      final goodFeatureColIdx = 0;
      final ignoredFeatureColIdx = 1;
      final bestFeatureColIdx = 2;

      final goodSplit = {
        MockTreeNode(): Matrix.fromList([
          [1, 2, 3, 4, -1, -1, -1],
        ]),
        MockTreeNode(): Matrix.fromList([
          [10, 20, 30, 40, -1, -1, -1],
        ]),
      };

      final bestSplit = {
        MockTreeNode(): Matrix.fromList([
          [15, 16, 17, 18, -1, 0, 0],
        ]),
        MockTreeNode(): Matrix.fromList([
          [150, 160, 170, 180, -1, 0, 0],
        ]),
      };

      final featuresIdxs = {goodFeatureColIdx, bestFeatureColIdx};

      final assessor = MockTreeSplitAssessor();
      final splitter = MockTreeSplitter();
      final categoricalValues = [1.0, 2.0, 3.0];
      final colIdxToUniqueValues = {ignoredFeatureColIdx: categoricalValues};

      when(splitter.split(observations, goodFeatureColIdx, targetColIdx))
          .thenReturn(goodSplit);
      when(splitter.split(observations, bestFeatureColIdx, targetColIdx))
          .thenReturn(bestSplit);

      when(assessor.getAggregatedError(goodSplit.values,
          targetColIdx)).thenReturn(0.51);
      when(assessor.getAggregatedError(bestSplit.values,
          targetColIdx)).thenReturn(0.1);

      final selector = GreedyTreeSplitSelector(assessor, splitter);
      final stump = selector.select(observations, targetColIdx,
          featuresIdxs, colIdxToUniqueValues);

      expect(stump.keys, equals(bestSplit.keys));
      expect(stump.values, equals(bestSplit.values));

      verifyNever(splitter.split(observations, ignoredFeatureColIdx,
          targetColIdx));
    });
  });
}
